@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
     Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
@@ -18,6 +18,7 @@
 
 class Character : public Rectangle
   {
+public:
   struct Guess
     {
     int code;
@@ -25,7 +26,8 @@ class Character : public Rectangle
     Guess( int c, int v ) throw() : code( c ), value( v ) {}
     };
 
-  std::vector< Blob * > bpv;		// the blobs forming this Character
+private:
+  std::vector< Blob * > blobpv;		// the blobs forming this Character
   std::vector< Guess > gv;		// vector of possible char codes
 					// and their associated values
 
@@ -38,7 +40,7 @@ class Character : public Rectangle
 
 public:
   Character( Blob * p ) throw()
-    : Rectangle( *p ), bpv( 1, p ) {}
+    : Rectangle( *p ), blobpv( 1, p ) {}
 
   Character( const Rectangle & re, int code, int value ) throw()
     : Rectangle( re ), gv( 1, Guess( code, value ) ) {}
@@ -50,7 +52,7 @@ public:
 
   int area() const throw();
   Blob & blob( int i ) throw();
-  int blobs() const throw() { return bpv.size(); }
+  int blobs() const throw() { return blobpv.size(); }
   Blob & main_blob() throw();
 
   void shift_blobp( Blob * p ) throw();
