@@ -1,5 +1,5 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
     Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
@@ -21,7 +21,7 @@ extern int verbosity;
 
 namespace Ocrad {
 
-void internal_error( const char * msg ) throw() __attribute__ ((noreturn));
+void internal_error( const char * const msg ) throw();
 bool similar( int a, int b, int percent_dif, int abs_dif = 1 ) throw();
 
 } // end namespace Ocrad
@@ -35,10 +35,11 @@ public:
   enum Value { ascii = 1, iso_8859_9 = 2, iso_8859_15 = 4 };
 
   Charset() throw() : charset_( 0 ) {}
-  bool enable( const char * name ) throw();
+  bool enable( const char * const name ) throw();
   bool enabled( Value cset ) const throw();
   bool only( Value cset ) const throw();
-  void show_error( const char * program_name, const char * arg ) const throw();
+  void show_error( const char * const program_name,
+                   const char * const arg ) const throw();
   };
 
 
@@ -51,9 +52,10 @@ private:
 
 public:
   Filter() throw() : type_( none ) {}
-  bool set( const char * name ) throw();
+  bool set( const char * const name ) throw();
   Type type() const throw() { return type_; }
-  void show_error( const char * program_name, const char * arg ) const throw();
+  void show_error( const char * const program_name,
+                   const char * const arg ) const throw();
   };
 
 
@@ -67,9 +69,10 @@ private:
 
 public:
   Transformation() throw() : type_( none ) {}
-  bool set( const char * name ) throw();
+  bool set( const char * const name ) throw();
   Type type() const throw() { return type_; }
-  void show_error( const char * program_name, const char * arg ) const throw();
+  void show_error( const char * const program_name,
+                   const char * const arg ) const throw();
   };
 
 
@@ -80,7 +83,7 @@ struct Control
   Charset charset;
   Filter filter;
   Format format;
-  FILE *outfile, *exportfile;
+  FILE * outfile, * exportfile;
   int debug_level;
   char filetype;
 
@@ -88,5 +91,5 @@ struct Control
     : format( byte ), outfile( stdout ), exportfile( 0 ),
       debug_level( 0 ), filetype( '4' ) {}
 
-  bool set_format( const char * name ) throw();
+  bool set_format( const char * const name ) throw();
   };
