@@ -1,9 +1,10 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006 Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+    Antonio Diaz Diaz.
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -12,8 +13,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
     Return values: 0 for a normal exit, 1 for environmental problems
@@ -39,72 +39,73 @@
 
 namespace {
 
-// Date of this version: 2006-10-20
-
 const char * invocation_name = 0;
 const char * const Program_name    = "GNU Ocrad";
 const char * const program_name    = "ocrad";
-const char * const program_version = "0.16";
-const char * const program_year    = "2006";
+const char * const program_year    = "2010";
 
 struct Input_control
   {
   Transformation transformation;
-  int rindex, layout_level, scale;
-  Rational threshold, ltrb[4];
-  char specialtype;
-  bool copy, crop, invert;
+  int scale;
+  Rational threshold, ltwh[4];
+  bool copy, cut, invert, layout;
 
   Input_control() throw()
-    : rindex( -1 ), layout_level( 0 ), scale( 0 ), threshold( -1 ),
-      specialtype( 0 ), copy( false ), crop( false ), invert( false ) {}
+    : scale( 0 ), threshold( -1 ),
+      copy( false ), cut( false ), invert( false ), layout( false ) {}
 
-  bool read_crop_rectangle( const char * ptr ) throw();
-  bool set_threshold( const char * ptr ) throw();
+  bool parse_cut_rectangle( const char * const s ) throw();
+  bool parse_threshold( const char * const s ) throw();
   };
 
 
-void show_error( const char * msg, const int errcode = 0, const bool help = false ) throw()
+void show_error( const char * const msg, const int errcode = 0,
+                 const bool help = false ) throw()
   {
-  if( msg && msg[0] != 0 )
+  if( verbosity >= 0 )
     {
-    std::fprintf( stderr, "%s: %s", program_name, msg );
-    if( errcode > 0 ) std::fprintf( stderr, ": %s", strerror( errcode ) );
-    std::fprintf( stderr, "\n" );
+    if( msg && msg[0] )
+      {
+      std::fprintf( stderr, "%s: %s", program_name, msg );
+      if( errcode > 0 ) std::fprintf( stderr, ": %s", std::strerror( errcode ) );
+      std::fprintf( stderr, "\n" );
+      }
+    if( help && invocation_name && invocation_name[0] )
+      std::fprintf( stderr, "Try `%s --help' for more information.\n",
+                    invocation_name );
     }
-  if( help && invocation_name && invocation_name[0] != 0 )
-    std::fprintf( stderr, "Try `%s --help' for more information.\n", invocation_name );
   }
 
 
-bool Input_control::read_crop_rectangle( const char * ptr ) throw()
+bool Input_control::parse_cut_rectangle( const char * const s ) throw()
   {
-  int c = ltrb[0].parse( ptr );				// left
-  if( c && ltrb[0] >= 0 && ptr[c] == ',' )
+  int c = ltwh[0].parse( s );				// left
+  if( c && s[c] == ',' )
     {
     int i = c + 1;
-    c = ltrb[1].parse( &ptr[i] );			// top
-    if( c && ltrb[1] >= 0 && ptr[i+c] == ',' )
+    c = ltwh[1].parse( &s[i] );				// top
+    if( c && s[i+c] == ',' )
       {
-      i += c + 1; c = ltrb[2].parse( &ptr[i] );		// right
-      if( c && ltrb[2] > 0 && ptr[i+c] == ',' )
+      i += c + 1; c = ltwh[2].parse( &s[i] );		// width
+      if( c && s[i+c] == ',' && ltwh[2] > 0 )
         {
-        i += c + 1; c = ltrb[3].parse( &ptr[i] );	// bottom
-        if( c && ltrb[3] > 0 ) { crop = true; return true; }
+        i += c + 1; c = ltwh[3].parse( &s[i] );		// height
+        if( c && ltwh[3] > 0 ) { cut = true; return true; }
         }
       }
     }
-  show_error( "invalid crop rectangle", 0, true );
+  show_error( "invalid cut rectangle.", 0, true );
   return false;
   }
 
 
-bool Input_control::set_threshold( const char * ptr ) throw()
+bool Input_control::parse_threshold( const char * const s ) throw()
   {
   Rational tmp;
-  if( tmp.parse( ptr ) && tmp >= 0 && tmp <= 1 )
+  if( tmp.parse( s ) && tmp >= 0 && tmp <= 1 )
     { threshold = tmp; return true; }
-  show_error( "threshold out of limits (0.0 - 1.0)", 0, true );
+  show_error( "threshold out of limits (0.0 - 1.0).", 0, true );
   return false;
   }
 
@@ -114,41 +115,43 @@ void show_help() throw()
   std::printf( "%s - Optical Character Recognition program.\n", Program_name );
   std::printf( "Reads pnm file(s), or standard input, and sends text to standard output.\n" );
   std::printf( "\nUsage: %s [options] [files]\n", invocation_name );
-  std::printf( "Options:\n" );
+  std::printf( "\nOptions:\n" );
   std::printf( "  -h, --help               display this help and exit\n" );
   std::printf( "  -V, --version            output version information and exit\n" );
   std::printf( "  -a, --append             append text to output file\n" );
-  std::printf( "  -b, --block=<n>          process only the specified text block\n" );
   std::printf( "  -c, --charset=<name>     try `--charset=help' for a list of names\n" );
   std::printf( "  -e, --filter=<name>      try `--filter=help' for a list of names\n" );
   std::printf( "  -f, --force              force overwrite of output file\n" );
   std::printf( "  -F, --format=<fmt>       output format (byte, utf8)\n" );
   std::printf( "  -i, --invert             invert image levels (white on black)\n" );
-  std::printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n" );
+  std::printf( "  -l, --layout             perform layout analysis\n" );
   std::printf( "  -o <file>                place the output into <file>\n" );
-  std::printf( "  -p, --crop=<l,t,r,b>     crop input image by given rectangle\n" );
+  std::printf( "  -q, --quiet              suppress all messages\n" );
   std::printf( "  -s, --scale=[-]<n>       scale input image by [1/]<n>\n" );
   std::printf( "  -t, --transform=<name>   try `--transform=help' for a list of names\n" );
   std::printf( "  -T, --threshold=<n%%>     threshold for binarization (0-100%%)\n" );
+  std::printf( "  -u, --cut=<l,t,w,h>      cut input image by given rectangle\n" );
   std::printf( "  -v, --verbose            be verbose\n" );
   std::printf( "  -x <file>                export OCR Results File to <file>\n" );
-  if( Ocrad::verbose )
+  if( verbosity > 0 )
     {
     std::printf( "  -1..6                    pnm output file type (debug)\n" );
     std::printf( "  -C, --copy               'copy' input to output (debug)\n" );
     std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n" );
-    std::printf( "  -S <type>                make a 'special file' (debug)\n" );
     }
   std::printf( "\nReport bugs to bug-ocrad@gnu.org\n" );
+  std::printf( "Ocrad home page: http://www.gnu.org/software/ocrad/ocrad.html\n" );
+  std::printf( "General help using GNU software: http://www.gnu.org/gethelp\n" );
   }
 
 
 void show_version() throw()
   {
-  std::printf( "%s version %s\n", Program_name, program_version );
+  std::printf( "%s %s\n", Program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
-  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
-  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
+  std::printf( "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n" );
+  std::printf( "This is free software: you are free to change and redistribute it.\n" );
+  std::printf( "There is NO WARRANTY, to the extent permitted by law.\n" );
   }
 
 
@@ -160,87 +163,55 @@ const char * my_basename( const char * filename ) throw()
   }
 
 
-int process_file( FILE *infile, const char * infile_name,
+int process_file( FILE * const infile, const char * const infile_name,
                   const Input_control & input_control,
                   const Control & control ) throw()
   {
-  if( Ocrad::verbose )
+  if( verbosity > 0 )
     std::fprintf( stderr, "processing file `%s'\n", infile_name );
   try
     {
-    Page_image page_image( infile, input_control.threshold, input_control.invert );
+    Page_image page_image( infile, input_control.invert );
 
-    if( input_control.crop &&
-        !page_image.crop( input_control.ltrb, input_control.threshold ) )
+    if( input_control.cut && !page_image.cut( input_control.ltwh ) )
       {
-      if( Ocrad::verbose )
-        std::fprintf( stderr, "file `%s' totally cropped out\n", infile_name );
+      if( verbosity > 0 )
+        std::fprintf( stderr, "file `%s' totally cut away\n", infile_name );
       return 1;
       }
+    page_image.threshold( input_control.threshold );
     page_image.transform( input_control.transformation );
     page_image.scale( input_control.scale );
-    page_image.analyse_layout( input_control.layout_level );
-    if( Ocrad::verbose )
-      std::fprintf( stderr, "number of text blocks = %d\n", page_image.zones() );
-    if( input_control.threshold < 0 ) page_image.adapt_thresholds();
 
-    if( input_control.rindex >= page_image.zones() )
-      {
-      std::fprintf( stderr,"This page has only %d text block(s)\n", page_image.zones() );
-      return 1;
-      }
-    if( input_control.specialtype != 0 )
-      {
-      if( control.outfile )
-        {
-        const char & t = input_control.specialtype;
-        if( t == 'v' || t == 'h' ) page_image.histogramize( t == 'v' );
-        else { show_error( "bad special type" ); return 1; }
-        page_image.save( control.outfile, control.filetype );
-        }
-      return 0;
-      }
     if( input_control.copy )
       {
-      if( control.outfile )
-        {
-        if( input_control.layout_level == 0 && page_image.zones() == 1 )
-          page_image.save( control.outfile, control.filetype );
-        else
-          for( int c = 0; c < page_image.zones(); ++c )
-            if( input_control.rindex < 0 || input_control.rindex == c )
-              page_image.save( control.outfile, control.filetype, c );
-        }
+      if( control.outfile ) page_image.save( control.outfile, control.filetype );
       return 0;
       }
 
-    Textpage textpage( page_image, my_basename( infile_name ), control );
+    Textpage textpage( page_image, my_basename( infile_name ), control,
+                       input_control.layout );
+    if( control.debug_level == 0 )
+      {
+      if( control.outfile ) textpage.print( control );
+      if( control.exportfile ) textpage.xprint( control );
+      }
     }
-  catch( Page_image::Error e ) { show_error( e.s ); return 2; }
-  if( Ocrad::verbose ) std::fprintf( stderr, "\n" );
+  catch( Page_image::Error e ) { show_error( e.msg ); return 2; }
+  if( verbosity > 0 ) std::fprintf( stderr, "\n" );
   return 0;
   }
 
 } // end namespace
 
 
-bool Ocrad::verbose = false;
-
-void Ocrad::internal_error( const char * msg ) throw()
-  {
-  std::string s( "internal error: " ); s += msg;
-  show_error( s.c_str() );
-  exit( 3 );
-  }
-
-
 // 'infile' contains the scanned image (in pnm format) to be converted
 // to text.
 // 'outfile' is the destination for the text version of the scanned
 // image. (or for a pnm file if debugging).
 // 'exportfile' is the Ocr Results File.
 //
-int main( int argc, char * argv[] ) throw()
+int main( const int argc, const char * const argv[] )
   {
   Input_control input_control;
   Control control;
@@ -248,7 +219,7 @@ int main( int argc, char * argv[] ) throw()
   bool append = false, force = false;
   invocation_name = argv[0];
 
-  static const Arg_parser::Option options[] =
+  const Arg_parser::Option options[] =
     {
     { '1', 0,           Arg_parser::no  },
     { '2', 0,           Arg_parser::no  },
@@ -257,7 +228,6 @@ int main( int argc, char * argv[] ) throw()
     { '5', 0,           Arg_parser::no  },
     { '6', 0,           Arg_parser::no  },
     { 'a', "append",    Arg_parser::no  },
-    { 'b', "block",     Arg_parser::yes },
     { 'c', "charset",   Arg_parser::yes },
     { 'C', "copy",      Arg_parser::no  },
     { 'D', "debug",     Arg_parser::yes },
@@ -266,13 +236,13 @@ int main( int argc, char * argv[] ) throw()
     { 'F', "format",    Arg_parser::yes },
     { 'h', "help",      Arg_parser::no  },
     { 'i', "invert",    Arg_parser::no  },
-    { 'l', "layout",    Arg_parser::yes },
+    { 'l', "layout",    Arg_parser::no  },
     { 'o', 0,           Arg_parser::yes },
-    { 'p', "crop",      Arg_parser::yes },
+    { 'q', "quiet",     Arg_parser::no  },
     { 's', "scale",     Arg_parser::yes },
-    { 'S', 0,           Arg_parser::yes },
     { 't', "transform", Arg_parser::yes },
     { 'T', "threshold", Arg_parser::yes },
+    { 'u', "cut",       Arg_parser::yes },
     { 'v', "verbose",   Arg_parser::no  },
     { 'V', "version",   Arg_parser::no  },
     { 'x', 0,           Arg_parser::yes },
@@ -282,12 +252,12 @@ int main( int argc, char * argv[] ) throw()
   if( parser.error().size() )				// bad option
     { show_error( parser.error().c_str(), 0, true ); return 1; }
 
-  int argind;
-  for( argind = 0; argind < parser.arguments(); ++argind )
+  int argind = 0;
+  for( ; argind < parser.arguments(); ++argind )
     {
     const int code = parser.code( argind );
     if( !code ) break;					// no more options
-    const char * arg = parser.argument( argind ).c_str();
+    const char * const arg = parser.argument( argind ).c_str();
     switch( code )
       {
       case '1':
@@ -296,35 +266,34 @@ int main( int argc, char * argv[] ) throw()
       case '4':
       case '5':
       case '6': control.filetype = code; break;
-      case 'C': input_control.copy = true; break;
-      case 'D': control.debug_level = std::strtol( arg, 0, 0 ); break;
-      case 'F': if( !control.set_format( arg ) )
-                  { show_error( "bad output format", 0, true ); return 1; }
-                break;
-      case 'S': input_control.specialtype = arg[0]; break;
-      case 'T': if( !input_control.set_threshold( arg ) ) return 1; break;
-      case 'V':	show_version(); return 0;
       case 'a': append = true; break;
-      case 'b': input_control.rindex = std::strtol( arg, 0, 0 ) - 1; break;
       case 'c': if( !control.charset.enable( arg ) )
                   { control.charset.show_error( program_name, arg ); return 1; }
                 break;
+      case 'C': input_control.copy = true; break;
+      case 'D': control.debug_level = std::strtol( arg, 0, 0 ); break;
       case 'e': if( !control.filter.set( arg ) )
                   { control.filter.show_error( program_name, arg ); return 1; }
                 break;
       case 'f': force = true; break;
+      case 'F': if( !control.set_format( arg ) )
+                  { show_error( "bad output format.", 0, true ); return 1; }
+                break;
       case 'h': show_help(); return 0;
       case 'i': input_control.invert = true; break;
-      case 'l': input_control.layout_level = std::strtol( arg, 0, 0 ); break;
-      case 'o':	outfile_name = arg; break;
-      case 'p': if( !input_control.read_crop_rectangle( arg ) ) return 1; break;
+      case 'l': input_control.layout = true; break;
+      case 'o': outfile_name = arg; break;
+      case 'q': verbosity = -1; break;
       case 's': input_control.scale = std::strtol( arg, 0, 0 ); break;
       case 't': if( !input_control.transformation.set( arg ) )
                   { input_control.transformation.show_error( program_name, arg );
                   return 1; }
                 break;
-      case 'v': Ocrad::verbose = true; break;
-      case 'x':	exportfile_name = arg; break;
+      case 'T': if( !input_control.parse_threshold( arg ) ) return 1; break;
+      case 'u': if( !input_control.parse_cut_rectangle( arg ) ) return 1; break;
+      case 'v': if( verbosity < 4 ) ++verbosity; break;
+      case 'V': show_version(); return 0;
+      case 'x': exportfile_name = arg; break;
       default : Ocrad::internal_error( "uncaught option" );
       }
     } // end process options
@@ -335,16 +304,19 @@ int main( int argc, char * argv[] ) throw()
     else if( force ) control.outfile = std::fopen( outfile_name, "w" );
     else if( ( control.outfile = std::fopen( outfile_name, "wx" ) ) == 0 )
       {
-      std::fprintf( stderr, "Output file %s already exists.\n", outfile_name );
+      if( verbosity >= 0 )
+        std::fprintf( stderr, "Output file %s already exists.\n", outfile_name );
       return 1;
       }
     if( !control.outfile )
-      { std::fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
+      {
+      if( verbosity >= 0 )
+        std::fprintf( stderr, "Cannot open %s\n", outfile_name );
+      return 1;
+      }
     }
 
-  if( exportfile_name && control.debug_level == 0 &&
-      input_control.scale == 0 && input_control.specialtype == 0 &&
-      !input_control.copy )
+  if( exportfile_name && control.debug_level == 0 && !input_control.copy )
     {
     if( std::strcmp( exportfile_name, "-" ) == 0 )
       { control.exportfile = stdout; if( !outfile_name ) control.outfile = 0; }
@@ -353,17 +325,18 @@ int main( int argc, char * argv[] ) throw()
       control.exportfile = std::fopen( exportfile_name, "w" );
       if( !control.exportfile )
         {
-        std::fprintf( stderr, "Cannot open %s\n", exportfile_name );
+        if( verbosity >= 0 )
+          std::fprintf( stderr, "Cannot open %s\n", exportfile_name );
         return 1;
         }
       }
     std::fprintf( control.exportfile,
                   "# Ocr Results File. Created by %s version %s\n",
-                  Program_name, program_version );
+                  Program_name, PROGVERSION );
     }
 
   // process any remaining command line arguments (input files)
-  FILE *infile = (argind < parser.arguments()) ? 0 : stdin;
+  FILE * infile = (argind < parser.arguments()) ? 0 : stdin;
   const char *infile_name = "-";
   int retval = 0;
   while( true )
@@ -374,9 +347,10 @@ int main( int argc, char * argv[] ) throw()
       if( argind >= parser.arguments() ) { infile = 0; break; }
       infile_name = parser.argument( argind++ ).c_str();
       if( std::strcmp( infile_name, "-" ) == 0 ) infile = stdin;
-      else infile = std::fopen( infile_name, "r" );
+      else infile = std::fopen( infile_name, "rb" );
       if( infile ) break;
-      std::fprintf( stderr, "Cannot open %s\n", infile_name );
+      if( verbosity >= 0 )
+        std::fprintf( stderr, "Cannot open %s\n", infile_name );
       if( retval == 0 ) retval = 1;
       }
     if( !infile ) break;
