@@ -273,8 +273,8 @@ const char * UCS::ucs_to_utf8( const int code ) throw()
 
   if( code < 0 || code > 0x7FFFFFFF ) { s[0] = 0; return s; } // invalid code
   if( code < 128 ) { s[0] = code; s[1] = 0; return s; }       // plain ascii
-  int i;
-  char mask;
+
+  int i, mask;
   if( code < 0x800 ) { i = 2; mask = 0xC0; }		// 110X XXXX
   else if( code < 0x10000 ) { i = 3; mask = 0xE0; }	// 1110 XXXX
   else if( code < 0x200000 ) { i = 4; mask = 0xF0; }	// 1111 0XXX
